import requests
import json
import click


@click.command()
@click.option('--task', type=click.Choice(['collect', 'metadata'], case_sensitive=False))
@click.option('--limit', default=100, help='number of elements per page')
@click.option('--fout', default='puppet_modules.json', help='File to save the modules data.')
@click.option('--fin', default='puppet_modules.json', help='File to save the modules data.')
def forge(task, limit, fout, fin):
    if task == 'collect':
        scrape_forge(fout, limit=limit)
    elif task == 'metadata':
        get_projects(fin, fout)


def get_projects(fin, fout):

    with open(fin) as f:
        modules = json.load(f)
    print(f'a total of {len(modules)} were collected')

    modules_info = dict()
    for module in modules:

        if module['owner']['username'] not in ('puppetlabs', 'puppet'):
            continue
    
        if module['deprecated_for']:
            continue

        if module['name'] not in modules_info:
            forge_href = f"https://forge.puppet.com/modules/{module['owner']['username']}/{module['name']}"
            modules_info[module['name']] = forge_href

    with open(fout, 'w') as f:
        json.dump(modules_info, f,  indent=4)


def save_to_json(fout, results):
    """Saves the modules info into a JSON
    file.

    Args:
        fout (string): output JSON file name
        results (JSON): modules data
    """
    with open(fout, 'w') as out:
        json.dump(results, out, indent=4)


def get_first_page(limit):
    """_summary_

    Returns:
        _type_: _description_
    """
    forge_path = f"https://forgeapi.puppet.com/v3/modules?limit={limit}"
    r = requests.get(forge_path)
    if r.status_code == 200:
        resp = r.json()
        modules = resp['pagination']['total']
        no_pages = modules // limit
        print(f"Number of total modules {modules}; number of pages {no_pages}")
        return no_pages, resp['results'], resp['pagination']['next'], 0


def scrape_forge(fout, limit):
    """_summary_

    Args:
        limit (int, optional): _description_. Defaults to 100.
    """
    no_pages, results, next_page, count = get_first_page(limit)
    for page in range(0, no_pages):
        r = requests.get(f"https://forgeapi.puppet.com{next_page}")
        if r.status_code == 200:
            resp = r.json()
            results += resp['results']
            next_page = resp['pagination']['next']
            print(
                f"Processing page {page+2}, adding more {limit} modules to the database... {next_page}")
            if count == 10:
                save_to_json(fout, results)
                print(f"Saved to file...")
                count = 0
            count += 1

    print(
        f"Process finished, {len(results)} modules information added to your database.")
    save_to_json(fout, results)


if __name__ == '__main__':
    forge()
