import argparse
import os
import subprocess
import json
import requests
import random

import pandas as pd
import numpy as np
from github import Github

import shlex
import subprocess
from pathlib import Path


def get_forms(file):
    """Function to process, clean and filter results
    from the practitioners validation.

    Args:
        file (json): files with the answers
    """
    df = pd.DataFrame(columns=['form_id', 'profession', 'security', 'infrastructure', 'prolific', 'q1_id',
                      'q1_type', 'q1_result', 'q2_id', 'q2_type', 'q2_result', 'q3_id', 'q3_type', 'q3_result'])

    with open(file) as json_file:
        forms = json.load(json_file)

    print(f"found a total of {len(forms)} entries")

    for entry in forms:
        form_data = {}
        form_data['form_id'] = entry['_id']
        form_data['prolific'] = entry['prolific']
        form_data['profession'] = entry['profession']
        form_data['security'] = entry['security']
        form_data['infrastructure'] = entry['infrastructure']

        count = 1
        for answer in entry['answers'][0]:
            form_data[f"q{count}_id"] = answer['_id']
            form_data[f"q{count}_type"] = answer['type']
            if 'result' not in answer.keys():
                form_data[f"q{count}_result"] = 'missing'
            else:
                form_data[f"q{count}_result"] = answer['result']
            count += 1

        df = df.append(form_data, ignore_index=True)

    print("cleaning participants that violate the experience requirement...")
    for idx, row in df.iterrows():
        if row['security'] == 0 and row['infrastructure'] == 0 and row['prolific'] == 'yes':
            df = df.drop(idx)
    
    print(f"- {len(df[df['prolific'] == 'no'])} practitioners were recruited through https://puppetcommunity.slack.com/ or r/puppet")
    print(f"- {len(df[df['prolific'] == 'yes'])} practitioners were recruited through prolific")

    df.to_csv('form-results.csv', index=False)


def get_comments(file):
    with open(file) as json_file:
        forms = json.load(json_file)

    doc = ''
    for form in forms:
        if form['security'] == 0 and form['infrastructure'] == 0 and form['prolific'] == 'yes':
            continue
        else:
            for answer in form['answers'][0]:
                if 'result' in answer.keys() and 'observations' in answer.keys():
                    if (answer['result'] == 'no' or answer['result'] == 'unsure') and answer['observations'] != '':
                        doc += answer['url'] + '\n' \
                            + answer['code'] + '\n' \
                            + 'Problem: ' + answer['type'] + '\n' \
                            + 'Answer: ' + answer['result'] + '\n'
                        if 'observations' in answer.keys():
                            doc += 'Observations: ' + \
                                answer['observations'] + '\n'
                        print(doc)
                        doc += '\n\n'
    with open("observations/false_positives.txt", "w") as text_file:
        text_file.write(doc)

    doc = ''
    for form in forms:
        if form['security'] == 0 and form['infrastructure'] == 0 and form['prolific'] == 'yes':
            continue
        else:
            for answer in form['answers'][0]:
                if 'result' in answer.keys() and 'observations' in answer.keys():
                    if answer['result'] == 'yes' and answer['observations'] != '':
                        doc += answer['url'] + '\n' \
                            + answer['code'] + '\n' \
                            + 'Problem: ' + answer['type'] + '\n' \
                            + 'Answer: ' + answer['result'] + '\n' \
                            + 'Observations: ' + \
                            answer['observations'] + '\n'
                        doc += '\n\n'
    with open("observations/true_positives.txt", "w") as text_file:
        text_file.write(doc)

def get_problems(file):
    with open(file) as json_file:
        forms = json.load(json_file)

    doc = ''
    for form in forms:
        if form['security'] == 0 and form['infrastructure'] == 0 and form['prolific'] == 'yes':
            continue
        else:
            for answer in form['answers'][0]:
                if 'result' in answer.keys() and 'observations' in answer.keys():
                    if answer['result'] == 'no' or answer['result'] == 'unsure':
                        doc += answer['url'] + '\n' \
                            + answer['code'] + '\n' \
                            + 'Problem: ' + answer['type'] + '\n' \
                            + 'Answer: ' + answer['result'] + '\n'
                        if 'observations' in answer.keys():
                            doc += 'Observations: ' + \
                                answer['observations'] + '\n'
                        print(doc)
                        doc += '\n\n'
    with open("problems/list.txt", "w") as text_file:
        text_file.write(doc)

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='Process Results')
    parser.add_argument('--collection', dest='collection',
                        choices=['form'])
    parser.add_argument('--export', dest='export',
                        choices=['results', 'comments', 'problems'])
    parser.add_argument('-file', type=str,
                        metavar='database file', help='base file')

    args = parser.parse_args()

    if args.collection == 'form':
        if args.export == 'results' and args.file:
            get_forms(file=args.file)
        elif args.export == 'comments' and args.file:
            get_comments(file=args.file)
        elif args.export == 'problems' and args.file:
            get_problems(file=args.file)
    else:
        print('Something is wrong. Verify your parameters')
