import pandas as pd
import os
import subprocess
import click
import json

checks = {
    'admin_by_default': 'DFLT_ADMN',
    'empty_password': 'EMPT_PASS',
    'hardcode_secret': 'HARD_CODE_SECR',
    'invalid_ip_addr_binding': 'BIND_USAG',
    'suspicious_comment': 'SUSP_COMM',
    'use_http_without_tls': 'HTTP_USAG',
    'use_of_weak_crypto_algorithm': 'MD5_USAG'
}

@click.command(name='download', help='download puppet files')
@click.argument('sample', nargs=1)
def download(sample):
    if sample in ('proportional', 'uniform'):
        df = pd.read_csv(f"validations/research_team_{sample}_results.csv")
    else:
        df = pd.read_csv(f"validations/{sample}_results.csv")
    
    fout_folder = f"puppet-files/{sample}/"
    if not os.path.exists(fout_folder):
        os.makedirs(fout_folder)


    for _, row in df.iterrows():
        url, _ = row['FILE_NAME'].split('#L')
        url = url.replace('blob', 'raw')
        out_path = url.replace('https://www.github.com/','').replace('/raw/','/').replace('/', '_')
    
        bashCommand = f"wget -O {fout_folder}/{out_path} {url}"
        process = subprocess.Popen(bashCommand.split(), stdout=subprocess.PIPE)
        process.communicate()
        
        
@click.command(name='check', help='check if alerts are detected')
@click.argument('sample', nargs=1)
@click.argument('tool', nargs=1)
def check(sample, tool):
    print(tool, sample)
    def load_alerts(tool, sample):
        with open(f'{tool}/{sample}.json') as json_file:
            return json.load(json_file)
        
    def check_alert(alerts, path_to_file, line, smell_type):
        for project in alerts:
            for alert in project:
                if path_to_file in alert['fullpath'] and int(line) == alert['line'] and checks[alert['check']] == smell_type:
                    return alert
         
    df = pd.read_csv(f"validations/research_team_{sample}_results.csv")
    alerts = load_alerts(tool, sample)
    
    for idx, row in df.iterrows():
        url, line = row['FILE_NAME'].split('#L')
        path_to_file = url.replace('blob', 'raw').replace('https://www.github.com/','').replace('https://github.com/','').replace('/raw/','/').replace('/', '_')
        alert = check_alert(alerts, path_to_file, line, row['SMELL_TYPE'])
        if alert:
            df.at[idx, 'found'] = 1
        else:
            df.at[idx, 'found'] = 0
    
    df['results'] = df['FINAL'].apply(lambda x: 'False Positive' if x else 'True Positive')
    print(df)
    
    # for idx, row in df[df['found'] != 1].iterrows():
    #     if not row['FINAL']:
    #         print(row['FILE_NAME'], row['SMELL_TYPE'], row['results'])
    
    for idx, row in df[df['results'] == 'False Positive'].iterrows():
        if not row['FINAL']:
            print(row['FILE_NAME'], row['SMELL_TYPE'], row['results'])
    
    df[df['found'] == 1].to_csv(f"{tool}/{sample}.csv")
    
    
@click.command(name='precision', help='generate precision table')
@click.argument('sample', nargs=1)
@click.argument('tool', nargs=1)
def precision(sample, tool):
    df = pd.read_csv(f"{tool}/{sample}.csv")
    
    df['results'] = df['FINAL'].apply(lambda x: '#FP' if x else '#TP')
    dfr = df.groupby('SMELL_TYPE')['results'].value_counts().to_frame()
    print(dfr)
    results = {'checker': [], '#TP': [], '#FP': [], 'prec': []}
    for _, row in dfr.iterrows():
        if row.name[0] not in results['checker']:
            results['checker'].append(row.name[0])
        if row.name[1] == '#FP':
            results['#FP'].append(row['results'])
        if row.name[1] == '#TP':
            results['#TP'].append(row['results'])
        if sample == 'proportional' and tool == 'slic' and row.name[0] == 'BIND_USAG':
            results['#FP'].append(0)
        if sample == 'proportional' and tool == 'infrasecure_v.0.1.0' and row.name[0] == 'BIND_USAG':
            results['#FP'].append(0)
        # if sample == 'proportional' and tool == 'infrasecure_v.0.1.0' and row.name[0] == 'MD5_USAG':
        #     results['#FP'].append(0)

    for i, _ in enumerate(results['checker']):
        total = results['#TP'][i] + results['#FP'][i]
        precision = results['#TP'][i] / total
        results['prec'].append("{:.2f}".format(precision))
    
    
    fdf = pd.DataFrame(results)
    fdf = pd.concat([fdf, pd.DataFrame({'checker': 'Total',
                                        "#TP": fdf['#TP'].sum(),
                                        "#FP": fdf['#FP'].sum(),
                                        "prec": "{:.2f}".format(fdf['#TP'].sum() / (fdf['#TP'].sum() + fdf['#FP'].sum())),
                                        },index=[7])])
    sort_dict = {'HARD_CODE_SECR':0,'HTTP_USAG':1,'SUSP_COMM':2,'MD5_USAG':3,'BIND_USAG':4,'EMPT_PASS':5,'DFLT_ADMN':6, 'Total': 7}
    fdf = fdf.iloc[fdf['checker'].map(sort_dict).sort_values().index]
    print(fdf)
    